@@ -1,4 +1,4 @@
-/*	$NetBSD: input.c,v 1.13 1999/07/25 00:24:38 hubertf Exp $	*/
+/*	$NetBSD: input.c,v 1.17 2005/02/15 12:56:20 jsm Exp $	*/
 
 /*-
  * Copyright (c) 1990, 1993
@@ -15,11 +15,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
+ * 3. Neither the name of the University nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -50,7 +46,7 @@
 #if 0
 static char sccsid[] = "@(#)input.c	8.1 (Berkeley) 5/31/93";
 #else
-__RCSID("$NetBSD: input.c,v 1.13 1999/07/25 00:24:38 hubertf Exp $");
+__RCSID("$NetBSD: input.c,v 1.17 2005/02/15 12:56:20 jsm Exp $");
 #endif
 #endif /* not lint */
 
@@ -71,7 +67,7 @@ typedef struct {
 	int	token;
 	int	to_state;
 	const char	*str;
-	const char	*(*func) __P((char));
+	const char	*(*func)(char);
 } RULE;
 
 typedef struct {
@@ -233,7 +229,7 @@ int
 getcommand()
 {
 	int	c, i, done;
-	const char	*s, *(*func) __P((char));
+	const char	*s, *(*func)(char);
 	PLANE	*pp;
 
 	rezero();
@@ -326,10 +322,10 @@ gettoken()
 						base = shell;
 					else
 						base++;
-					execl(shell, base, 0);
+					execl(shell, base, (char *) 0);
 				}
 				else
-					execl(_PATH_BSHELL, "sh", 0);
+					execl(_PATH_BSHELL, "sh", (char *) 0);
 
 				exit(0);	/* oops */
 			}
