@@ -1,18 +1,44 @@
-/*	$NetBSD: hunt.c,v 1.12 2001/02/05 00:40:45 christos Exp $	*/
+/*	$NetBSD: hunt.c,v 1.23 2004/11/05 21:30:32 dsl Exp $	*/
 /*
- *  Hunt
- *  Copyright (c) 1985 Conrad C. Huang, Gregory S. Couch, Kenneth C.R.C. Arnold
- *  San Francisco, California
+ * Copyright (c) 1983-2003, Regents of the University of California.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions are 
+ * met:
+ * 
+ * + Redistributions of source code must retain the above copyright 
+ *   notice, this list of conditions and the following disclaimer.
+ * + Redistributions in binary form must reproduce the above copyright 
+ *   notice, this list of conditions and the following disclaimer in the 
+ *   documentation and/or other materials provided with the distribution.
+ * + Neither the name of the University of California, San Francisco nor 
+ *   the names of its contributors may be used to endorse or promote 
+ *   products derived from this software without specific prior written 
+ *   permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS 
+ * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
+ * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <sys/cdefs.h>
 #ifndef lint
-__RCSID("$NetBSD: hunt.c,v 1.12 2001/02/05 00:40:45 christos Exp $");
+__RCSID("$NetBSD: hunt.c,v 1.23 2004/11/05 21:30:32 dsl Exp $");
 #endif /* not lint */
 
 # include	<sys/param.h>
 # include	<sys/stat.h>
 # include	<sys/time.h>
+# include	<sys/poll.h>
 # include	<ctype.h>
 # include	<err.h>
 # include	<errno.h>
@@ -25,6 +51,7 @@ __RCSID("$NetBSD: hunt.c,v 1.12 2001/02/05 00:40:45 christos Exp $");
 static struct termios saved_tty;
 # endif
 # include	<unistd.h>
+# include	<ifaddrs.h>
 
 # include	"hunt.h"
 
@@ -48,10 +75,6 @@ static struct termios saved_tty;
 # define	put_str		addstr
 # endif
 
-#if !defined(BSD_RELEASE) || BSD_RELEASE < 44
-extern int	_putchar();
-#endif
-
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
 #endif
@@ -89,13 +112,15 @@ static int	in_visual;
 
 extern int	cur_row, cur_col;
 
-void	dump_scores __P((SOCKET));
-long	env_init __P((long));
-void	fill_in_blanks __P((void));
-void	leave __P((int, const char *)) __attribute__((__noreturn__));
-int	main __P((int, char *[]));
+void	dump_scores(SOCKET);
+long	env_init(long);
+void	fill_in_blanks(void);
+void	leave(int, const char *) __attribute__((__noreturn__));
+void	leavex(int, const char *) __attribute__((__noreturn__));
+void	fincurs(void);
+int	main(int, char *[]);
 # ifdef INTERNET
-SOCKET *list_drivers __P((void));
+SOCKET *list_drivers(void);
 # endif
 
 extern int	Otto_mode;
@@ -124,7 +149,7 @@ main(ac, av)
 			break;
 		case 't':
 			team = *optarg;
-			if (!isdigit(team)) {
+			if (!isdigit((unsigned char)team)) {
 				warnx("Team names must be numeric");
 				team = ' ';
 			}
@@ -260,13 +285,11 @@ main(ac, av)
 # endif /* !USE_CURSES */
 	in_visual = TRUE;
 	if (LINES < SCREEN_HEIGHT || COLS < SCREEN_WIDTH)
-		leave(1, "Need a larger window");
+		leavex(1, "Need a larger window");
 	clear_the_screen();
 	(void) signal(SIGINT, intr);
 	(void) signal(SIGTERM, sigterm);
-#ifdef SIGEMT
-	(void) signal(SIGEMT, sigemt);
-#endif
+	(void) signal(SIGUSR1, sigusr1);
 	(void) signal(SIGPIPE, SIG_IGN);
 #if !defined(USE_CURSES) && defined(SIGTSTP)
 	(void) signal(SIGTSTP, tstp);
@@ -277,7 +300,7 @@ main(ac, av)
 		find_driver(TRUE);
 
 		if (Daemon.sin_port == 0)
-			leave(1, "Game not found, try again");
+			leavex(1, "Game not found, try again");
 
 	jump_in:
 		do {
@@ -296,7 +319,6 @@ main(ac, av)
 			if (connect(Socket, (struct sockaddr *) &Daemon,
 			    DAEMON_SIZE) < 0) {
 				if (errno != ECONNREFUSED) {
-					warn("connect");
 					leave(1, "connect");
 				}
 			}
@@ -322,8 +344,7 @@ main(ac, av)
 		(void) strcpy(Daemon.sun_path, Sock_name);
 		if (connect(Socket, &Daemon, DAEMON_SIZE) < 0) {
 			if (errno != ENOENT) {
-				warn("connect");
-				leave(1, "connect2");
+				leavex(1, "connect2");
 			}
 			start_driver();
 
@@ -352,7 +373,7 @@ main(ac, av)
 		if ((enter_status = quit(enter_status)) == Q_QUIT)
 			break;
 	}
-	leave(0, (char *) NULL);
+	leavex(0, (char *) NULL);
 	/* NOTREACHED */
 	return(0);
 }
@@ -364,26 +385,32 @@ broadcast_vec(s, vector)
 	int			s;		/* socket */
 	struct	sockaddr	**vector;
 {
-	char			if_buf[BUFSIZ];
-	struct	ifconf		ifc;
-	struct	ifreq		*ifr;
-	unsigned int		n;
 	int			vec_cnt;
+	struct ifaddrs		*ifp, *ip;
 
 	*vector = NULL;
-	ifc.ifc_len = sizeof if_buf;
-	ifc.ifc_buf = if_buf;
-	if (ioctl(s, SIOCGIFCONF, (char *) &ifc) < 0)
+	if (getifaddrs(&ifp) < 0)
 		return 0;
+
 	vec_cnt = 0;
-	n = ifc.ifc_len / sizeof (struct ifreq);
-	*vector = (struct sockaddr *) malloc(n * sizeof (struct sockaddr));
+	for (ip = ifp; ip; ip = ip->ifa_next)
+		if ((ip->ifa_addr->sa_family == AF_INET) &&
+		    (ip->ifa_flags & IFF_BROADCAST))
+			vec_cnt++;
+
+	*vector = (struct sockaddr *)
+		malloc(vec_cnt * sizeof(struct sockaddr_in));
 	if (*vector == NULL)
 		leave(1, "Out of memory!");
-	for (ifr = ifc.ifc_req; n != 0; n--, ifr++)
-		if (ioctl(s, SIOCGIFBRDADDR, ifr) >= 0)
-			memcpy(&(*vector)[vec_cnt++], &ifr->ifr_addr,
-				sizeof (struct sockaddr));
+
+	vec_cnt = 0;
+	for (ip = ifp; ip; ip = ip->ifa_next)
+		if ((ip->ifa_addr->sa_family == AF_INET) &&
+		    (ip->ifa_flags & IFF_BROADCAST))
+			memcpy(&(*vector)[vec_cnt++], ip->ifa_broadaddr,
+			       sizeof(struct sockaddr_in));
+
+	freeifaddrs(ifp);
 	return vec_cnt;
 }
 # endif
@@ -411,20 +438,19 @@ list_drivers()
 	static	SOCKET		*listv;
 	static	unsigned int	listmax;
 	unsigned int		listc;
-	fd_set			mask;
-	struct timeval		wait;
+	struct pollfd		set[1];
 
 	if (initial) {			/* do one time initialization */
 # ifndef BROADCAST
 		sethostent(1);		/* don't bother to close host file */
 # endif
 		if (gethostname(local_name, sizeof local_name) < 0) {
-			leave(1, "Sorry, I have no name.");
+			leavex(1, "Sorry, I have no name.");
 			/* NOTREACHED */
 		}
 		local_name[sizeof(local_name) - 1] = '\0';
 		if ((hp = gethostbyname(local_name)) == NULL) {
-			leave(1, "Can't find myself.");
+			leavex(1, "Can't find myself.");
 			/* NOTREACHED */
 		}
 		local_address = * ((struct in_addr *) hp->h_addr);
@@ -438,7 +464,6 @@ list_drivers()
 
 	test_socket = socket(SOCK_FAMILY, SOCK_DGRAM, 0);
 	if (test_socket < 0) {
-		warn("socket");
 		leave(1, "socket system call failed");
 		/* NOTREACHED */
 	}
@@ -448,7 +473,7 @@ list_drivers()
 
 	if (Sock_host != NULL) {	/* explicit host given */
 		if ((hp = gethostbyname(Sock_host)) == NULL) {
-			leave(1, "Unknown host");
+			leavex(1, "Unknown host");
 			/* NOTREACHED */
 		}
 		test.sin_addr = *((struct in_addr *) hp->h_addr);
@@ -465,20 +490,13 @@ list_drivers()
 
 # ifdef BROADCAST
 	if (initial)
-		brdc = broadcast_vec(test_socket, (struct sockaddr **) &brdv);
-
-	if (brdc <= 0) {
-		initial = FALSE;
-		test.sin_addr = local_address;
-		goto test_one_host;
-	}
+		brdc = broadcast_vec(test_socket, (void *) &brdv);
 
 # ifdef SO_BROADCAST
 	/* Sun's will broadcast even though this option can't be set */
 	option = 1;
 	if (setsockopt(test_socket, SOL_SOCKET, SO_BROADCAST,
 	    &option, sizeof option) < 0) {
-		warn("setsockopt broadcast");
 		leave(1, "setsockopt broadcast");
 		/* NOTREACHED */
 	}
@@ -490,11 +508,16 @@ list_drivers()
 		test.sin_addr = brdv[i].sin_addr;
 		if (sendto(test_socket, (char *) &msg, sizeof msg, 0,
 		    (struct sockaddr *) &test, DAEMON_SIZE) < 0) {
-			warn("sendto");
 			leave(1, "sendto");
 			/* NOTREACHED */
 		}
 	}
+	test.sin_addr = local_address;
+	if (sendto(test_socket, (char *) &msg, sizeof msg, 0,
+	    (struct sockaddr *) &test, DAEMON_SIZE) < 0) {
+		leave(1, "sendto");
+		/* NOTREACHED */
+	}
 # else /* !BROADCAST */
 	/* loop thru all hosts on local net and send msg to them. */
 	msg = htons(C_TESTMSG());
@@ -512,9 +535,9 @@ list_drivers()
 get_response:
 	namelen = DAEMON_SIZE;
 	errno = 0;
+	set[0].fd = test_socket;
+	set[0].events = POLLIN;
 	for (;;) {
-		wait.tv_sec = 1;
-		wait.tv_usec = 0;
 		if (listc + 1 >= listmax) {
 			listmax += 20;
 			listv = (SOCKET *) realloc((char *) listv,
@@ -523,9 +546,7 @@ get_response:
 				leave(1, "Out of memory!");
 		}
 
-		FD_ZERO(&mask);
-		FD_SET(test_socket, &mask);
-		if (select(test_socket + 1, &mask, NULL, NULL, &wait) == 1 &&
+		if (poll(set, 1, 1000) == 1 &&
 		    recvfrom(test_socket, (char *) &port_num, sizeof(port_num),
 		    0, (struct sockaddr *) &listv[listc], &namelen) > 0) {
 			/*
@@ -543,8 +564,7 @@ get_response:
 		}
 
 		if (errno != 0 && errno != EINTR) {
-			warn("select/recvfrom");
-			leave(1, "select/recvfrom");
+			leave(1, "poll/recvfrom");
 			/* NOTREACHED */
 		}
 
@@ -666,7 +686,7 @@ start_driver()
 
 # ifdef MONITOR
 	if (Am_monitor) {
-		leave(1, "No one playing.");
+		leavex(1, "No one playing.");
 		/* NOTREACHED */
 	}
 # endif
@@ -689,7 +709,6 @@ start_driver()
 	refresh();
 	procid = fork();
 	if (procid == -1) {
-		warn("fork");
 		leave(1, "fork failed.");
 	}
 	if (procid == 0) {
@@ -705,9 +724,7 @@ start_driver()
 			execl(Driver, "HUNT", "-p", use_port, (char *) NULL);
 # endif
 		/* only get here if exec failed */
-#ifdef SIGEMT
-		(void) kill(getppid(), SIGEMT);	/* tell mom */
-#endif
+		(void) kill(getppid(), SIGUSR1);	/* tell mom */
 		_exit(1);
 	}
 # ifdef USE_CURSES
@@ -729,7 +746,7 @@ start_driver()
 void
 bad_con()
 {
-	leave(1, "The game is full.  Sorry.");
+	leavex(1, "The game is full.  Sorry.");
 	/* NOTREACHED */
 }
 
@@ -740,7 +757,7 @@ bad_con()
 void
 bad_ver()
 {
-	leave(1, "Version number mismatch. No go.");
+	leavex(1, "Version number mismatch. No go.");
 	/* NOTREACHED */
 }
 
@@ -752,20 +769,20 @@ SIGNAL_TYPE
 sigterm(dummy)
 	int dummy __attribute__((__unused__));
 {
-	leave(0, (char *) NULL);
+	leavex(0, (char *) NULL);
 	/* NOTREACHED */
 }
 
 
 /*
- * sigemt:
- *	Handle a emt signal - shouldn't happen on vaxes(?)
+ * sigusr1:
+ *	Handle a usr1 signal
  */
 SIGNAL_TYPE
-sigemt(dummy)
+sigusr1(dummy)
 	int dummy __attribute__((__unused__));
 {
-	leave(1, "Unable to start driver.  Try again.");
+	leavex(1, "Unable to start driver.  Try again.");
 	/* NOTREACHED */
 }
 
@@ -833,7 +850,7 @@ intr(dummy)
 				(void) write(Socket, "q", 1);
 				(void) close(Socket);
 			}
-			leave(0, (char *) NULL);
+			leavex(0, (char *) NULL);
 		}
 		else if (ch == 'n') {
 			(void) signal(SIGINT, intr);
@@ -857,15 +874,7 @@ intr(dummy)
 	}
 }
 
-/*
- * leave:
- *	Leave the game somewhat gracefully, restoring all current
- *	tty stats.
- */
-void
-leave(eval, mesg)
-	int	eval;
-	const char	*mesg;
+void fincurs()
 {
 	if (in_visual) {
 # ifdef USE_CURSES
@@ -884,9 +893,36 @@ leave(eval, mesg)
 		_puts(TE);
 # endif /* !USE_CURSES */
 	}
-	if (mesg != NULL)
-		puts(mesg);
-	exit(eval);
+}
+
+/*
+ * leave:
+ *	Leave the game somewhat gracefully, restoring all current
+ *	tty stats.
+ */
+void
+leave(eval, mesg)
+	int	eval;
+	const char	*mesg;
+{
+	int serrno = errno;
+	fincurs();
+	errno = serrno;
+	err(eval, mesg ? mesg : "");
+}
+
+/*
+ * leave:
+ *	Leave the game somewhat gracefully, restoring all current
+ *	tty stats.
+ */
+void
+leavex(eval, mesg)
+	int	eval;
+	const char	*mesg;
+{
+	fincurs();
+	errx(eval, mesg ? mesg : "");
 }
 
 #if !defined(USE_CURSES) && defined(SIGTSTP)
@@ -1027,7 +1063,7 @@ env_init(enter_status)
 # endif
 			else if (strncmp(envp, "team=", s - envp + 1) == 0) {
 				team = *(s + 1);
-				if (!isdigit(team))
+				if (!isdigit((unsigned char)team))
 					team = ' ';
 				if ((s = strchr(envp, ',')) == NULL) {
 					*envp = '\0';
@@ -1090,7 +1126,7 @@ again:
 		goto again;
 	}
 	for (cp = name; *cp != '\0'; cp++)
-		if (!isprint(*cp)) {
+		if (!isprint((unsigned char)*cp)) {
 			name[0] = '\0';
 			printf("Illegal character in your code name.\n");
 			goto again;
